#define WITHOUT_CRC 0x00
#define WITH_CRC 0x01
#define ADD_CRC 0x02
#define MODE_RX 0x80

#define N_MAX 64
unsigned char ucSt[N_MAX];

void setup() {
  Serial.begin(9600);
}

// Compute the MODBUS RTU CRC
// Adapted to Arduino from http://www.ccontrolsys.com/w/How_to_Compute_the_Modbus_RTU_Message_CRC
unsigned int uiModRTU_CRC(byte* buf, int len){
  unsigned int crc = 0xFFFF;
 
  for (int pos = 0; pos < len; pos++) {
    crc ^= (unsigned int)buf[pos];          // XOR byte into least sig. byte of crc
 
    for (int i = 8; i != 0; i--) {    // Loop over each bit
      if ((crc & 0x0001) != 0) {      // If the LSB is set
        crc >>= 1;                    // Shift right and XOR 0xA001
        crc ^= 0xA001;
      }
      else                            // Else LSB is not set
        crc >>= 1;                    // Just shift right
    }
  }
  // Note, this number has low and high bytes swapped, so use it accordingly (or swap bytes)
  return crc;  
}

void vAnswer(byte* ucSt,int len){
  int i;
  
  for(i = 0; i < len ; i++)
    Serial.write(ucSt[i]);
}

void vAnswerCheckingCRC(byte* ucSt,int len){
  int i;
  unsigned int uiCRC = uiModRTU_CRC(ucSt,len-2);

  if((byte)(uiCRC >> 8) == ucSt[len-1] && (byte)(uiCRC & 0xFF) == ucSt[len-2]){ 
    for(i = 0; i < len-2 ; i++)
      Serial.write(ucSt[i]);
  }else{
    Serial.println("CRC doesn't match");
  }
}

void vAnswerAddingCRC(byte* ucSt,int len){
  int i;
  unsigned int uiCRC = uiModRTU_CRC(ucSt,len);
  
  for(i = 0; i < len ; i++)
    Serial.write(ucSt[i]);
  Serial.write((byte)(uiCRC & 0xFF));
  Serial.write((byte)(uiCRC >> 8));
}

void vProcessa(byte* ucSt,int len){
  if(ucSt[0] & MODE_RX){            // 1st byte is function mode
    Serial.println("Mode Rx");
  }else{
    if(ucSt[1] == len-2){           // 2nd byte is payload length
      switch(ucSt[0]){
        case WITHOUT_CRC:
             vAnswer(ucSt+2,ucSt[1]);
             break;
        case WITH_CRC:
             vAnswerCheckingCRC(ucSt+2,ucSt[1]);
             break;
        case ADD_CRC:
             vAnswerAddingCRC(ucSt+2,ucSt[1]);
             break;
        default:
             vAnswer(ucSt,len);
     }
    }else
      Serial.println("Length doesn't match");
  }
}

void loop() {
  int nCmpt = 0;
  
  while(Serial.available() > 0) {
    ucSt[nCmpt] = (unsigned char)Serial.read();
    nCmpt++;
    delay(2);
  }
  if(nCmpt){
    vProcessa(ucSt,nCmpt);
    Serial.println("");
  }
  nCmpt = 0;
}

