#!/usr/bin/python
# -*- coding: utf-8 -*-

"""
dealingBinary.py

http://binefa.cat/blog/
https://twitter.com/JordiBinefa
20160605
"""

import sys
from PyQt4 import Qt, QtGui, QtCore
from ui_lora import Ui_LoraGUI
import serial
from time import sleep
from crc16b import calcByte,calcString,hexCRC,checkModbusWithCrc,readHex

removeCRC = 0x00
addCRC = 0x01
CRCatMicroCtrl = 0x02

class LoraGUI(QtGui.QWidget, Ui_LoraGUI):
	def __init__(self, parent=None, f=QtCore.Qt.WindowFlags()):
		QtGui.QWidget.__init__(self, parent, f)
		self.setupUi(self)
		
		if len( sys.argv ) == 2 :
			szPort = sys.argv[1]
			nBauds = 9600
		else :
			if len( sys.argv ) == 3 :
				szPort = sys.argv[1]
				nBauds = int(sys.argv[2])
			else :
				szPort = "/dev/ttyUSB0"
				nBauds = 9600

		port = szPort
		baudrate = nBauds
		szTitle = "%s @%d" % (port,baudrate)	
		self.bConnPort = True
		try: 	
			self.ser = serial.Serial(
				port,
				baudrate,
				timeout=0,
				parity=serial.PARITY_NONE,
				stopbits=serial.STOPBITS_TWO,
				bytesize=serial.EIGHTBITS
			)
		except serial.SerialException as e:
			print szPort, " port is not connected"
			self.close()
			szTitle = "%s is NOT connected" % port
			self.bConnPort = False
			#sys.exit(-1)
		self.setWindowTitle(szTitle)
		self.connect(self.lineEditStr, Qt.SIGNAL("textChanged(QString)"), self.funcStr2Hex)
		self.connect(self.lineEditHex, Qt.SIGNAL("textChanged(QString)"), self.funcHexChanged)
		self.connect(self.radioButtonWithoutCRC, Qt.SIGNAL("toggled(bool)"), self.funcRemoveCRC)
		self.connect(self.radioButtonAddCRC, Qt.SIGNAL("toggled(bool)"), self.funcAddCRC)
		self.connect(self.radioButtonCrcAtMicroCtrl, Qt.SIGNAL("toggled(bool)"), self.funcCRCuC)	
		self.connect(self.radioButtonTx, Qt.SIGNAL("toggled(bool)"), self.funcTx)
		self.connect(self.radioButtonRx, Qt.SIGNAL("clicked(bool)"), self.funcRx)		
		self.connect(self.btTxRx, Qt.SIGNAL("clicked()"), self.btTxRxPressed)
		self.temporitzador = QtCore.QTimer()
		self.connect(self.temporitzador, Qt.SIGNAL("timeout()"), self.funcioTemporitzada)
		self.temporitzador.start(100)
		self.byCtrl = removeCRC

	def funcAddCRC(self,b):
		if b:
			self.byCtrl = addCRC

	def funcRemoveCRC(self,b):
		if b:
			self.byCtrl = removeCRC
			
	def funcCRCuC(self,b):
		if b:
			self.byCtrl = CRCatMicroCtrl			
			
	def funcTx(self,b):
		if b:
			self.btTxRx.setEnabled(True)

	def funcRx(self,b):
		if b:
			self.btTxRx.setEnabled(False)
		st0 = "%02X" % (self.byCtrl	| 0x80 )
		szHtml = "<font color=\"orange\">" + st0 + "</font>"
		self.plainTextEdit.appendHtml(szHtml)
		st0 = "%02X" % (self.byCtrl | 0x80)	# 1st byte is function mode
		st1 = "%02X" % 0	    			# 2nd byte is payload length
		self.funcSerialWrite(st0 + st1)
			
	def funcHexChanged(self,szHex):
		nSzHex = len(szHex)
		if nSzHex % 2 == 0:
			i = 0
			szNum = ""
			nByteArray = []
			while ( i < nSzHex ):
				szByte = "%c%c" % (str(szHex[i])[0],str(szHex[i+1])[0])
				szNum += szByte
				nByte = int (szByte,16)
				nByteArray.append(nByte);
				i += 2
			hCRC = hexCRC(nByteArray)
			szAddedCRC="%02X%02X"%(hCRC>>8,hCRC&0xFF)
			self.lineEditCRC.setText(szAddedCRC)
			self.lineEditHex.setText(szNum)

	def funcStr2Hex(self,szStr):
		st = ""
		for ch in str(szStr):
			st += "%02X" % ord(ch)					
		self.lineEditHex.setText(st)

	def funcSerialWrite(self,sz):
		nCmpt = 0
		st = ""
		stMB = ""
		for ch in str(sz):
			st += ch
			nCmpt = nCmpt + 1
			if nCmpt == 2:
				nSt = int(str(st),16)
				stMB+="%c"%(nSt)
				nCmpt = 0
				st = ""
		#readHex(stMB)
		#print ""
		self.ser.write(stMB)
		self.bConn = 1
		
	def btTxRxPressed(self):
		if self.lineEditHex.text().length() % 2 == 0:
			szHtml = QtCore.QString("")
			nSzHex = self.lineEditHex.text().length() / 2
			if self.byCtrl == addCRC:
				nSzHex += 2
			st0 = "%02X" % self.byCtrl	# 1st byte is function mode
			st1 = "%02X" % nSzHex	    # 2nd byte is payload length
			szHtml += "<font color=\"orange\">" + st0 + "</font>"
			szHtml += "<font color=\"red\">" + st1 + "</font>"
			szHtml += "<font color=\"orange\">" + self.lineEditHex.text() + "</font>"
			stBytes = st0 + st1 + self.lineEditHex.text()
			if self.byCtrl == addCRC:
				szHtml += "<font color=\"red\">" + self.lineEditCRC.text() + "</font>"
				stBytes += self.lineEditCRC.text()
			self.plainTextEdit.appendHtml(szHtml)
			self.funcSerialWrite(stBytes)

	def processa(self,sz,nQ):
		#print "Rebut:\n",sz
		#print "He rebut %d bytes" % nQ
		szHtml = QtCore.QString("<font color=\"blue\">")
		#for i in range(0, len(sz)):
		for i in range(0, nQ):
			szHtml += "{:02X}".format(ord(sz[i]))
		szHtml += "</font>"
		if self.checkBoxReadCharString.isChecked():
			if self.byCtrl == CRCatMicroCtrl:
				szHtml += "<font color=\"BlueViolet\">  ("+ sz[:-4] + ")</font>"
			else:
				szHtml += "<font color=\"BlueViolet\">  ("+ sz[:-2] + ")</font>"		
		self.plainTextEdit.appendHtml(szHtml)
			
	def funcioTemporitzada(self):
		if self.bConnPort:
			data = self.ser.read(1)
			n = self.ser.inWaiting()
			if n:
				data = data + self.ser.read(n)
			if len(data):
				self.processa(data,n-1)
	
	def __del__(self):
		if self.bConnPort:
			self.ser.close()

if __name__ == '__main__':

	app = QtGui.QApplication(sys.argv)
	window = LoraGUI()
	window.show()

	sys.exit(app.exec_())
